unit FtpUnit;

(*
 * From: http://synapse.ararat.cz/files/contrib/ftpdemo.zip
 *
 * v1 - ?
 * v2 - 01.04.2003 Written by Krzysztof Guc nakielan@poczta.fm
 * v3 - 2010-04-14 Added ProgressBar - Arnulf Sortland - http://S-A.no/
 * v4 -
 *)

interface

uses
  SysUtils, Types, TypInfo, Classes, Variants, QTypes, QGraphics, QControls,
  QForms, QDialogs, QStdCtrls, FTPsend, DNSsend, QComCtrls, SynaMisc, blcksock;

type

  TftpForm = class(TForm)
    EditFtpServer: TEdit;
    EditPort: TEdit;
    EditUser: TEdit;
    EditPassw: TEdit;
    EditRemoteFile: TEdit;
    EditLocalFile: TEdit;
    MemoLog: TMemo;
    ButtonPutFile: TButton;
    ButtonGetFile: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label8: TLabel;
    ProgressBar: TProgressBar;
    procedure ButtonPutFileClick(Sender: TObject);
    procedure ButtonGetFileClick(Sender: TObject);
    procedure SockPutCallBack(Sender: TObject;
      Reason: THookSocketReason; const Value: string);
    procedure SockGetCallBack(Sender: TObject;
      Reason: THookSocketReason; const Value: string);
  private
    TotalBytes: integer;                // Total FileSize
    CurrentBytes: integer;              // Get/ Put
    { Private declarations }
  public
    { Public declarations }
  end;

var
  ftpForm: TftpForm;

implementation

{$R *.xfm}

uses myutils;

// Searching host in all nameservers

function myhost2ip(host: string): string;
var
  dnsy, dns: string;
  ipaddrs: TstringList;
  dnsnr: byte;
  found: boolean;
const
  dnsqtype = 1;
begin
  dnsnr := 0;
  found := false;
  dnsy := GetDNS;
  ipaddrs := TstringList.Create;
  // Searching host in all nameservers
  repeat
    inc(dnsnr);
    ipaddrs.Clear;
    // Searching host in one nameserver
    dns := myGetnArg(dnsy, dnsnr, ',');
    if dns = '' then
      break;
    with TDNSSend.Create do
    try
      TargetHost := dns;
      found := DNSQuery(host, dnsqtype, ipaddrs);
    finally
      Free;
    end;
    // Break if find;
    if found = true then
      break;
  until false;
  // Return host
  if found = true then
    Result := ipaddrs[0];
end;

procedure TftpForm.ButtonPutFileClick(Sender: TObject);
var
  ip: string;
  ftp: TFTPSend;
  f: file of Byte;
begin
  ProgressBar.Position := 0;
  MemoLog.Clear;
  MemoLog.Lines.Add('Loking up:     ' + EditFtpServer.Text + '...');

  ip := myhost2ip(EditFtpServer.Text);
  if ip <> '' then
    MemoLog.lines.Add('Connecting:     ' + ip + ':' + EditPort.Text + '...')
  else
  begin
    MemoLog.Lines.Add('Cannot find:' + '  ' + EditFtpServer.text);
    exit;
  end;

  AssignFile(f, EditLocalFile.Text);
  Reset(f);
  try
    TotalBytes := FileSize(f);
  finally
    CloseFile(f);
  end;

  ftp := TFTPSend.Create;
  try
    ftp.DSock.OnStatus := SockPutCallBack; // Update progressbar
    ftp.Username := EditUser.text;
    ftp.Password := EditPassw.text;
    ftp.TargetHost := EditFtpServer.text;
    ftp.TargetPort := EditPort.text;

    if ftp.Login then
      MemoLog.Lines.Add('Login:           success')
    else
    begin
      MemoLog.Lines.Add('Login:           incorrect');
      exit;
    end;

    // Delete old file, before sending new
    ftp.FTPCommand('DELE ' + EditRemoteFile.Text);
    ftp.DirectFileName := EditLocalFile.Text;
    ftp.DirectFile := true;

    MemoLog.Lines.Add('Sending file:   ' + EditLocalFile.Text);
    MemoLog.Lines.Add('TotalBytes:   ' + IntToStr(TotalBytes));

    if ftp.StoreFile(EditRemoteFile.Text, false) = true then
      MemoLog.Lines.Add('Transfer completed')
    else
      MemoLog.lines.add('Transfer failed');

    ftp.Logout;
    MemoLog.Lines.Add('Connection closed');
  finally
    ftp.Free;
  end;
end;

procedure TftpForm.SockPutCallBack(Sender: TObject;
  Reason: THookSocketReason; const Value: string);
begin
  case Reason of
    HR_WriteCount:
      begin
        inc(CurrentBytes, StrToIntDef(Value, 0));
        ProgressBar.Position := Round(1000 * (CurrentBytes / TotalBytes));
      end;
    HR_Connect: CurrentBytes := 0;
  end;
end;

procedure TftpForm.SockGetCallBack(Sender: TObject;
  Reason: THookSocketReason; const Value: string);
// var  v: string;
begin
  case Reason of
    HR_ReadCount:
      begin
        inc(CurrentBytes, StrToIntDef(Value, 0));
        ProgressBar.Position := Round(1000 * (CurrentBytes / TotalBytes));
      end;
    HR_Connect: CurrentBytes := 0;
    (*
      else
        begin
          v := getEnumName(typeInfo(THookSocketReason),
            integer(Reason)) + ' ' + Value);
          MemoLog.Lines.Add(v);
        end;
    *)
  end;
end;

procedure TftpForm.ButtonGetFileClick(Sender: TObject);
var
  ip: string;
  ftp: TFTPSend;
begin
  ProgressBar.Position := 0;
  MemoLog.Clear;
  MemoLog.Lines.Add('Loking up:     ' + EditFtpServer.Text + '...');

  ip := myhost2ip(EditFtpServer.Text);
  if ip <> '' then
    MemoLog.lines.Add('Connecting:     ' + ip + ':' + EditPort.Text + '...')
  else
  begin
    MemoLog.Lines.Add('Cannot find:' + '  ' + EditFtpServer.text);
    exit;
  end;

  ftp := TFTPSend.Create;
  try
    ftp.DSock.OnStatus := SockGetCallBack; // Update progressbar
    ftp.Username := EditUser.text;
    ftp.Password := EditPassw.text;
    ftp.TargetHost := EditFtpServer.text;
    ftp.TargetPort := EditPort.text;

    if ftp.Login then
      MemoLog.Lines.Add('Login:           success')
    else
    begin
      MemoLog.Lines.Add('Login:           incorrect');
      exit;
    end;

    ftp.DirectFileName := EditLocalFile.Text;
    ftp.DirectFile := true;
    TotalBytes := ftp.FileSize(EditRemoteFile.Text);

    MemoLog.Lines.Add('Load file:  ' + EditRemoteFile.Text);
    MemoLog.Lines.Add('TotalBytes: ' + IntToStr(TotalBytes));

    if ftp.RetrieveFile(EditRemoteFile.Text, false) = true then
      MemoLog.Lines.Add('Transfer completed')
    else
      MemoLog.lines.add('Transfer failed');

    ftp.Logout;
    MemoLog.Lines.Add('Connection closed');
  finally
    ftp.Free;
  end;
end;

end.


